import astrometry
import json
import logging
import pathlib

dirname = pathlib.Path(__file__).resolve().parent

logging.getLogger().setLevel(logging.INFO)

solver = astrometry.Solver(
    astrometry.series_4100.index_files(
        cache_directory=dirname / "astrometry_cache",
        scales={7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19},
    )
)

# x, y, radius (pixels)
with open(dirname / "apod4_stars.json") as stars_file:
    stars = sorted(json.load(stars_file), key=lambda star: -star[2])

print("Solve with default parameters")
solution = solver.solve(
    stars=stars,
    size_hint=astrometry.SizeHint(
        lower_arcsec_per_pixel=163.0,
        upper_arcsec_per_pixel=172.0,
    ),
    position_hint=astrometry.PositionHint(ra_deg=185.8, dec_deg=55.5, radius_deg=10.0),
    solution_parameters=astrometry.SolutionParameters(
        logodds_callback=lambda logodds_list: astrometry.Action.STOP,
    ),
)
if solution.has_match():
    for star in solution.best_match().stars:
        print(star)
else:
    print(
        "Astrometry could not find a match.",
        "Reduce output_logodds_threshold in solution_parameters,",
        "or use another set of index files",
    )



import astrometry
import json
import logging
import pathlib

dirname = pathlib.Path(__file__).resolve().parent

logging.getLogger().setLevel(logging.INFO)

solver = astrometry.Solver(
    astrometry.series_5000.index_files(
        cache_directory=dirname / "astrometry_cache",
        scales={5},
    )
)

with open(dirname / "astrosat_stars.json") as stars_file:
    stars = json.load(stars_file)

print("Solve with default parameters")
solution = solver.solve(
    stars=stars,
    size_hint=astrometry.SizeHint(
        lower_arcsec_per_pixel=0.40,
        upper_arcsec_per_pixel=0.43,
    ),
    position_hint=astrometry.PositionHint(
        ra_deg=53.0523, dec_deg=-27.8858, radius_deg=0.33
    ),
    solution_parameters=astrometry.SolutionParameters(
        sip_order=0,
        tune_up_logodds_threshold=None,
        output_logodds_threshold=20.0,
        logodds_callback=lambda logodds_list: astrometry.Action.STOP,
        slices_generator=astrometry.batches_generator(50),
    ),
)
if solution.has_match():
    for star in solution.best_match().stars:
        print(star)
else:
    print(
        "Astrometry could not find a match.",
        "Reduce output_logodds_threshold in solution_parameters,",
        "or use another set of index files",
    )



from __future__ import annotations
import astrometry_extension
import dataclasses
import enum
import math
import operator
import pathlib
import threading
import typing
from .series import CHUNK_SIZE as CHUNK_SIZE
from .series import DOWNLOAD_SUFFIX as DOWNLOAD_SUFFIX
from .series import TIMEOUT as TIMEOUT
from .series import size_to_string as size_to_string
from .series import Series as Series
from .series import series_4100 as series_4100
from .series import series_4200 as series_4200
from .series import series_5000 as series_5000
from .series import series_5200 as series_5200
from .series import series_5200_heavy as series_5200_heavy
from .series import series_6000 as series_6000
from .series import series_6100 as series_6100


DEFAULT_LOWER_ARCSEC_PER_PIXEL = 0.1
DEFAULT_UPPER_ARCSEC_PER_PIXEL = 1000.0


class SupportsFloatMapping(typing.Protocol):
    def __getitem__(self, index: typing.SupportsIndex, /) -> typing.SupportsFloat:
        ...


@dataclasses.dataclass
class SizeHint:
    lower_arcsec_per_pixel: float
    upper_arcsec_per_pixel: float

    def __post_init__(self):
        assert self.lower_arcsec_per_pixel > 0
        assert self.upper_arcsec_per_pixel > 0
        assert self.lower_arcsec_per_pixel <= self.upper_arcsec_per_pixel


@dataclasses.dataclass
class PositionHint:
    ra_deg: float
    dec_deg: float
    radius_deg: float

    def __post_init__(self):
        assert self.ra_deg >= 0.0 and self.ra_deg < 360.0
        assert self.dec_deg >= -90.0 and self.dec_deg <= 90.0
        assert self.radius_deg >= 0


class Action(enum.IntEnum):
    STOP = 0
    CONTINUE = 1

    def __bool__(self):
        return self == Action.CONTINUE


class Parity(enum.IntEnum):
    NORMAL = 0
    FLIP = 1
    BOTH = 2


def batches_generator(
    batch_size: int,
) -> typing.Callable[[int], typing.Iterable[tuple[int, int]]]:
    def slices_generator(stars_count: int) -> typing.Iterable[tuple[int, int]]:
        if stars_count < batch_size:
            yield (0, stars_count)
        for start in range(0, stars_count - batch_size + 1, batch_size):
            yield (start, start + batch_size)

    return slices_generator


@dataclasses.dataclass
class SolutionParameters:
    solve_id: typing.Optional[str] = None
    uniformize_index: bool = True  # uniformize field stars at the matched index scale before verifying a match
    deduplicate: bool = True  # de-duplicate field stars before verifying a match
    sip_order: int = 3  # 0 means "no SIP distortion"
    sip_inverse_order: int = 0  # 0 means "same as sip_order"
    distance_from_quad_bonus: bool = True  # Assume that stars far from the matched quad will have larger positional variance?
    positional_noise_pixels: float = 1.0
    distractor_ratio: float = 0.25
    code_tolerance_l2_distance: float = 0.01
    minimum_quad_size_pixels: typing.Optional[
        float
    ] = None  # None means "use minimum_quad_size_fraction"
    minimum_quad_size_fraction: float = 0.1
    maximum_quad_size_pixels: float = 0.0  # 0.0 means "infinity"
    maximum_quads: int = 0  # number of field quads to try, 0 means no limit
    maximum_matches: int = 0  # number of quad matches to try, 0 means no limit
    parity: Parity = Parity.BOTH
    tune_up_logodds_threshold: typing.Optional[float] = 14.0  # None means "no tune-up"
    output_logodds_threshold: float = 21.0
    slices_generator: typing.Callable[
        [int], typing.Iterable[tuple[int, int]]
    ] = batches_generator(25)
    logodds_callback: typing.Callable[[list[float]], Action] = lambda _: Action.CONTINUE

    def __post_init__(self):
        assert self.sip_order >= 0
        assert self.sip_inverse_order >= 0
        assert self.positional_noise_pixels >= 0.0
        assert self.distractor_ratio > 0.0 and self.distractor_ratio <= 1.0
        assert self.code_tolerance_l2_distance >= 0.0
        assert (
            self.minimum_quad_size_pixels is None
            or self.minimum_quad_size_pixels >= 0.0
        )
        assert self.minimum_quad_size_fraction >= 0.0
        assert self.maximum_quad_size_pixels >= 0.0
        assert self.sip_order > 0 or self.tune_up_logodds_threshold is None


@dataclasses.dataclass
class Star:
    ra_deg: float
    dec_deg: float
    metadata: dict[str, typing.Any]


@dataclasses.dataclass
class Match:
    logodds: float
    center_ra_deg: float
    center_dec_deg: float
    scale_arcsec_per_pixel: float
    index_path: pathlib.Path
    stars: tuple[Star, ...]
    quad_stars: tuple[Star, ...]
    wcs_fields: dict[str, tuple[typing.Any, str]]


@dataclasses.dataclass
class Solution:
    solve_id: str
    matches: list[Match]

    def __post_init__(self):
        if len(self.matches) > 0:
            self.matches.sort(key=operator.attrgetter("logodds"), reverse=True)

    def has_match(self) -> bool:
        return len(self.matches) > 0

    def best_match(self) -> Match:
        return self.matches[0]


class Solver(astrometry_extension.Solver):
    def __init__(self, index_files: list[pathlib.Path]):
        super().__init__([str(path.resolve()) for path in index_files])
        self.solve_id_lock = threading.Lock()
        self.solve_id = 0

    def solve(
        self,
        stars: typing.Iterable[SupportsFloatMapping],
        size_hint: typing.Optional[SizeHint],
        position_hint: typing.Optional[PositionHint],
        solution_parameters: SolutionParameters,
    ) -> Solution:
        with self.solve_id_lock:
            self.solve_id += 1

        stars_xs: list[float] = []
        stars_ys: list[float] = []
        for star in stars:
            stars_xs.append(float(star[0]))
            stars_ys.append(float(star[1]))
        slices_starts: list[int] = []
        slices_ends: list[int] = []
        for star_slice in solution_parameters.slices_generator(len(stars_xs)):
            assert star_slice[0] >= 0
            assert star_slice[1] <= len(stars_xs)
            assert star_slice[0] < star_slice[1]
            slices_starts.append(star_slice[0])
            slices_ends.append(star_slice[1])
        assert len(slices_starts) > 0
        solve_id = (
            str(self.solve_id)
            if solution_parameters.solve_id is None
            else solution_parameters.solve_id
        )
        if size_hint is None:
            size_hint = SizeHint(
                lower_arcsec_per_pixel=DEFAULT_LOWER_ARCSEC_PER_PIXEL,
                upper_arcsec_per_pixel=DEFAULT_UPPER_ARCSEC_PER_PIXEL,
            )
        if solution_parameters.minimum_quad_size_pixels is None:
            star_x_minimum = math.inf
            star_x_maximum = 0.0
            for star_x in stars_xs:
                star_x_minimum = min(star_x_minimum, star_x)
                star_x_maximum = max(star_x_maximum, star_x)
            star_y_minimum = math.inf
            star_y_maximum = 0.0
            for star_y in stars_ys:
                star_y_minimum = min(star_y_minimum, star_y)
                star_y_maximum = max(star_y_maximum, star_y)
            extent = min(
                star_x_maximum - star_x_minimum, star_y_maximum - star_y_minimum
            )
            if extent < 0.0:
                minimum_quad_size_pixels = 0.0
            else:
                minimum_quad_size_pixels = (
                    solution_parameters.minimum_quad_size_fraction * extent
                )
        else:
            minimum_quad_size_pixels = solution_parameters.minimum_quad_size_pixels
        assert (
            solution_parameters.maximum_quad_size_pixels == 0.0
            or minimum_quad_size_pixels < solution_parameters.maximum_quad_size_pixels
        )
        raw_solution = super().solve(
            stars_xs if isinstance(stars_xs, list) else list(stars_xs),
            stars_ys if isinstance(stars_ys, list) else list(stars_ys),
            size_hint.lower_arcsec_per_pixel,
            size_hint.upper_arcsec_per_pixel,
            None
            if position_hint is None
            else (
                position_hint.ra_deg,
                position_hint.dec_deg,
                position_hint.radius_deg,
            ),
            solve_id,
            solution_parameters.uniformize_index,
            solution_parameters.deduplicate,
            solution_parameters.sip_order,
            solution_parameters.sip_inverse_order,
            solution_parameters.distance_from_quad_bonus,
            solution_parameters.positional_noise_pixels,
            solution_parameters.distractor_ratio,
            solution_parameters.code_tolerance_l2_distance,
            minimum_quad_size_pixels,
            solution_parameters.maximum_quad_size_pixels,
            solution_parameters.maximum_quads,
            solution_parameters.maximum_matches,
            int(solution_parameters.parity),
            solution_parameters.tune_up_logodds_threshold,
            solution_parameters.output_logodds_threshold,
            slices_starts,
            slices_ends,
            solution_parameters.logodds_callback,
        )
        if raw_solution is None:
            return Solution(solve_id=solve_id, matches=[])
        stars = {
            key: Star(ra_deg=ra_deg, dec_deg=dec_deg, metadata=metadata)
            for key, (ra_deg, dec_deg, metadata) in raw_solution[0].items()
        }
        solution = Solution(
            solve_id=solve_id,
            matches=[
                Match(
                    logodds=logodds,
                    center_ra_deg=center_ra_deg,
                    center_dec_deg=center_dec_deg,
                    scale_arcsec_per_pixel=scale_arcsec_per_pixel,
                    index_path=pathlib.Path(index_path),
                    stars=tuple(stars[key] for key in stars_keys),
                    quad_stars=tuple(stars[key] for key in quad_stars_keys),
                    wcs_fields=wcs_fields,
                )
                for (
                    logodds,
                    center_ra_deg,
                    center_dec_deg,
                    scale_arcsec_per_pixel,
                    index_path,
                    stars_keys,
                    quad_stars_keys,
                    wcs_fields,
                ) in raw_solution[1]
            ],
        )
        return solution



import pathlib
import platform
import setuptools
import setuptools.command.build_py
import setuptools.extension

dirname = pathlib.Path(__file__).resolve().parent

with open(dirname / "README.md") as file:
    long_description = file.read()

extra_compile_args = []
extra_link_args = []
define_macros = []
include_dirs = [
    "astrometry.net/gsl-an",
    "astrometry.net/include",
    "astrometry.net/include/astrometry",
    "astrometry.net/util",
]
sources = [
    # gsl
    "astrometry.net/gsl-an/blas/blas.c",
    "astrometry.net/gsl-an/block/init.c",
    *(
        str(path.as_posix())
        for path in sorted((dirname / "astrometry.net" / "gsl-an" / "cblas").iterdir())
        if path.suffix == ".c" and path.name != "hypot.c"
    ),
    "astrometry.net/gsl-an/err/error.c",
    "astrometry.net/gsl-an/err/stream.c",
    "astrometry.net/gsl-an/err/strerror.c",
    "astrometry.net/gsl-an/linalg/bidiag.c",
    "astrometry.net/gsl-an/linalg/householder.c",
    "astrometry.net/gsl-an/linalg/lu.c",
    "astrometry.net/gsl-an/linalg/qr.c",
    "astrometry.net/gsl-an/linalg/svd.c",
    "astrometry.net/gsl-an/matrix/copy.c",
    "astrometry.net/gsl-an/matrix/init.c",
    "astrometry.net/gsl-an/matrix/matrix.c",
    "astrometry.net/gsl-an/matrix/rowcol.c",
    "astrometry.net/gsl-an/matrix/submatrix.c",
    "astrometry.net/gsl-an/matrix/swap.c",
    "astrometry.net/gsl-an/matrix/view.c",
    "astrometry.net/gsl-an/permutation/init.c",
    "astrometry.net/gsl-an/permutation/permutation.c",
    "astrometry.net/gsl-an/permutation/permute.c",
    "astrometry.net/gsl-an/sys/coerce.c",
    "astrometry.net/gsl-an/sys/fdiv.c",
    "astrometry.net/gsl-an/sys/infnan.c",
    "astrometry.net/gsl-an/vector/copy.c",
    "astrometry.net/gsl-an/vector/init.c",
    "astrometry.net/gsl-an/vector/oper.c",
    "astrometry.net/gsl-an/vector/subvector.c",
    "astrometry.net/gsl-an/vector/swap.c",
    "astrometry.net/gsl-an/vector/vector.c",
    # libkd
    "astrometry.net/libkd/kdint_ddd.c",
    "astrometry.net/libkd/kdint_dds.c",
    "astrometry.net/libkd/kdint_ddu.c",
    "astrometry.net/libkd/kdint_dss.c",
    "astrometry.net/libkd/kdint_duu.c",
    "astrometry.net/libkd/kdint_fff.c",
    "astrometry.net/libkd/kdint_lll.c",
    "astrometry.net/libkd/kdtree.c",
    "astrometry.net/libkd/kdtree_dim.c",
    "astrometry.net/libkd/kdtree_fits_io.c",
    # qfits
    "astrometry.net/qfits-an/anqfits.c",
    "astrometry.net/qfits-an/qfits_byteswap.c",
    "astrometry.net/qfits-an/qfits_card.c",
    "astrometry.net/qfits-an/qfits_convert.c",
    "astrometry.net/qfits-an/qfits_error.c",
    "astrometry.net/qfits-an/qfits_float.c",
    "astrometry.net/qfits-an/qfits_header.c",
    "astrometry.net/qfits-an/qfits_memory.c",
    "astrometry.net/qfits-an/qfits_rw.c",
    "astrometry.net/qfits-an/qfits_table.c",
    "astrometry.net/qfits-an/qfits_time.c",
    "astrometry.net/qfits-an/qfits_tools.c",
    # solver
    "astrometry.net/solver/quad-utils.c",
    "astrometry.net/solver/solver.c",
    "astrometry.net/solver/tweak2.c",
    "astrometry.net/solver/verify.c",
    # util
    "astrometry.net/util/an-endian.c",
    "astrometry.net/util/bl.c",
    "astrometry.net/util/codekd.c",
    "astrometry.net/util/datalog.c",
    "astrometry.net/util/errors.c",
    "astrometry.net/util/fit-wcs.c",
    "astrometry.net/util/fitsbin.c",
    "astrometry.net/util/fitsfile.c",
    "astrometry.net/util/fitsioutils.c",
    "astrometry.net/util/fitstable.c",
    "astrometry.net/util/gslutils.c",
    "astrometry.net/util/healpix.c",
    "astrometry.net/util/index.c",
    "astrometry.net/util/ioutils.c",
    "astrometry.net/util/log.c",
    "astrometry.net/util/matchobj.c",
    "astrometry.net/util/mathutil.c",
    "astrometry.net/util/permutedsort.c",
    "astrometry.net/util/quadfile.c",
    "astrometry.net/util/sip.c",
    "astrometry.net/util/sip-utils.c",
    "astrometry.net/util/starkd.c",
    "astrometry.net/util/starutil.c",
    "astrometry.net/util/starxy.c",
    "astrometry.net/util/tic.c",
    # extension
    "astrometry_extension/astrometry_extension.c",
]

compiler = platform.python_compiler()
if compiler.startswith("Clang") or compiler.startswith("GCC"):
    extra_compile_args += ["-Wno-sign-compare"]


class BuildCommand(setuptools.command.build_py.build_py):
    def run(self):
        with open(
            dirname
            / "astrometry.net"
            / "include"
            / "astrometry"
            / "os-features-config.h",
            "w",
        ) as output:
            output.write(
                "\n".join(
                    (
                        "#define HAVE_NETPBM 0",
                        '#define AN_GIT_REVISION "0.89-7-g47849f04"',
                        '#define AN_GIT_DATE "Fri Jan 28 16:25:01 2022 -0500"',
                        '#define AN_GIT_URL "https://github.com/dstndstn/astrometry.net"\n',
                    )
                )
            )
        with open(dirname / "astrometry.net" / "gsl-an" / "config.h", "w") as output:
            output.write(
                "\n".join(
                    (
                        "#define GSL_DISABLE_DEPRECATED 1",
                        "#define HAVE_DECL_ACOSH 1",
                        "#define HAVE_DECL_ASINH 1",
                        "#define HAVE_DECL_ATANH 1",
                        "#define HAVE_DECL_EXPM1 1",
                        "#define HAVE_DECL_FEENABLEEXCEPT 0",
                        "#define HAVE_DECL_FESETTRAPENABLE 0",
                        "#define HAVE_DECL_FINITE 1",
                        "#define HAVE_DECL_FREXP 1",
                        "#define HAVE_DECL_HYPOT 1",
                        "#define HAVE_DECL_ISFINITE 1",
                        "#define HAVE_DECL_ISINF 1",
                        "#define HAVE_DECL_ISNAN 1",
                        "#define HAVE_DECL_LDEXP 1",
                        "#define HAVE_DECL_LOG1P 1",
                        "#define HAVE_DLFCN_H 1",
                        "#define HAVE_EXIT_SUCCESS_AND_FAILURE 1",
                        "#define HAVE_EXTENDED_PRECISION_REGISTERS 1",
                        "#define HAVE_INTTYPES_H 1",
                        "#define HAVE_LIBM 1",
                        "#define HAVE_MEMCPY 1",
                        "#define HAVE_MEMMOVE 1",
                        "#define HAVE_MEMORY_H 1",
                        "#define HAVE_PRINTF_LONGDOUBLE 1",
                        "#define HAVE_STDINT_H 1",
                        "#define HAVE_STDLIB_H 1",
                        "#define HAVE_STRDUP 1",
                        "#define HAVE_STRINGS_H 1",
                        "#define HAVE_STRING_H 1",
                        "#define HAVE_STRTOL 1",
                        "#define HAVE_STRTOUL 1",
                        "#define HAVE_SYS_STAT_H 1",
                        "#define HAVE_SYS_TYPES_H 1",
                        "#define HAVE_UNISTD_H 1",
                        "#define HAVE_VPRINTF 1",
                        '#define LT_OBJDIR ".libs/"',
                        '#define PACKAGE "gsl"',
                        '#define PACKAGE_BUGREPORT ""',
                        '#define PACKAGE_NAME "gsl"',
                        '#define PACKAGE_STRING "gsl 1.11"',
                        '#define PACKAGE_TARNAME "gsl"',
                        '#define PACKAGE_VERSION "1.11"',
                        "#define RELEASED",
                        "#define STDC_HEADERS 1",
                        '#define VERSION "1.11"',
                        "#if !HAVE_EXIT_SUCCESS_AND_FAILURE",
                        "#define EXIT_SUCCESS 0",
                        "#define EXIT_FAILURE 1",
                        "#endif",
                        "#if HAVE_EXTENDED_PRECISION_REGISTERS",
                        "#define GSL_COERCE_DBL(x) (gsl_coerce_double(x))",
                        "#else",
                        "#define GSL_COERCE_DBL(x) (x)",
                        "#endif",
                        "#if !HAVE_DECL_HYPOT",
                        "#define hypot gsl_hypot",
                        "#endif",
                        "#if !HAVE_DECL_LOG1P",
                        "#define log1p gsl_log1p",
                        "#endif",
                        "#if !HAVE_DECL_EXPM1",
                        "#define expm1 gsl_expm1",
                        "#endif",
                        "#if !HAVE_DECL_ACOSH",
                        "#define acosh gsl_acosh",
                        "#endif",
                        "#if !HAVE_DECL_ASINH",
                        "#define asinh gsl_asinh",
                        "#endif",
                        "#if !HAVE_DECL_ATANH",
                        "#define atanh gsl_atanh",
                        "#endif",
                        "#if !HAVE_DECL_LDEXP",
                        "#define ldexp gsl_ldexp",
                        "#endif",
                        "#if !HAVE_DECL_FREXP",
                        "#define frexp gsl_frexp",
                        "#endif",
                        "#if !HAVE_DECL_ISINF",
                        "#define isinf gsl_isinf",
                        "#endif",
                        "#if !HAVE_DECL_ISFINITE",
                        "#define isfinite gsl_finite",
                        "#endif",
                        "#if !HAVE_DECL_FINITE",
                        "#define finite gsl_finite",
                        "#endif",
                        "#if !HAVE_DECL_ISNAN",
                        "#define isnan gsl_isnan",
                        "#endif",
                        "#ifdef __GNUC__",
                        "#define DISCARD_POINTER(p) do { ; } while(p ? 0 : 0);",
                        "#else",
                        "#define DISCARD_POINTER(p)",
                        "#endif",
                        "#if defined(GSL_RANGE_CHECK_OFF) || !defined(GSL_RANGE_CHECK)",
                        "#define GSL_RANGE_CHECK 0",
                        "#endif",
                        "#define RETURN_IF_NULL(x) if (!x) { return ; }\n",
                    )
                )
            )
        super().run()


setuptools.setup(
    name="astrometry",
    version="4.0.0",
    url="https://github.com/neuromorphicsystems/astrometry",
    author="ICNS, Alexandre Marcireau",
    author_email="alexandre.marcireau@gmail.com",
    description="Astrometry.net solver interface",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=["requests"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    cmdclass={
        "build_py": BuildCommand,
    },
    ext_modules=[
        setuptools.extension.Extension(
            "astrometry_extension",
            language="c",
            sources=sources,
            include_dirs=include_dirs,
            libraries=[],
            extra_compile_args=extra_compile_args,
            extra_link_args=extra_link_args,
            define_macros=define_macros,
        ),
    ],
    packages=["astrometry"],
)
